package RebuildAt1stView;

use strict;
use base qw( MT::App );

use MT;
use MT::Template;
use MT::WeblogPublisher;

@RebuildAt1stView::ISA = qw(MT::App);

sub init {
	my $app = shift;
	$app->SUPER::init_request(@_);
	$app->add_methods( main => \&view );
	$app->{default_mode} = 'main';
	$app->{requires_login} = 0;
	$app;
}

my $mt = MT->new(Config => '../../mt-config.cgi');

sub view {
    my $app = shift;
    my $blog_id;
    my $url = $app->param('uri') || $ENV{"REDIRECT_URL"}; # or $app->path_info ?;
    my $cfg = $app->config;
    require MT::Blog;
    my $blog = MT::Blog->load( undef, { limit => 1 } )
        or return $app->error($app->translate('Loading blog failed'));
    my $org_uri = $url;
    my $idx = $cfg->IndexBasename || 'index';
    my $ext = $blog->file_extension || 'html';
    $idx .= '.' . $ext if $ext ne '';
    my @urls = ( $url );
    if ($url =~ /\/$/) {
        $url .= $idx;
    }
    require MT::FileInfo;
    my $fi;
    my @fis = MT::FileInfo->load({  url  => $url },
                                 {  sort => 'id',
                                    direction => 'ascend' });
    my $i = 1;
    if ( @fis > 1 ) {
        for my $finfo ( @fis ) {
            if ($i == 1) {
                $fi = $finfo;
            } else {
                $finfo->remove
                    or return $app->trans_error('Error removing fileinfo: [_1]', $finfo->errstr);
            }
        }
        $i++;
    }
    unless ( defined $fi ) {
        $fi = MT::FileInfo->load({
                                    url => $url,
                                  },
                                  { limit => 1, }
                                 ); 
    }
    if ( $fi ) {
        $blog_id = $fi->blog_id;
        my $pub = MT::WeblogPublisher->new;
        my $res = $pub->rebuild_from_fileinfo($fi);
        my $fp = $fi->file_path;
        if ( ( $res ) || ( -f $fp ) ) {
            open ( FH, $fp );
            my $html;
            while ( <FH> ) {
                $html .= $_;
            }
            close ( FH );
            return $html;
        } else {
            # re-try
            my $at = $fi->archive_type;
            my $tmpl_id = $fi->template_id;
            my $template = MT::Template->load( { id => $tmpl_id } );
            unless ( defined $template ) {
                return $app->error($app->translate('Loading template failed'));
            } else {
                if ( $template->build_dynamic ) {
                    return $app->error($app->translate('Template is dynamic'));
                }
            }
            if ( $at eq 'index' ) {
                $pub->rebuild_indexes( {
                    BlogID => $blog_id,
                    Force => 1,
                    Template => $tmpl_id,
                });
            } else {
                my $entry;
                if ( $fi->entry_id ) {
                    $entry = MT::Entry->load( { id => $fi->entry_id } );
                    if ( defined $entry ) {
                        if ( $entry-> status == 2 ) {
                            $pub->rebuild_entry( {
                                Entry  => $entry,
                                Blog   => MT::Blog->load( { id => $entry->blog_id } ),
                                BuildDependencies => 0,
                            });
                        } else {
                            return $app->error($app->translate('Entey is not released'));
                        }
                    }
                } else {
                    my ( $start, $end );
                    my $blog = MT::Blog->load( { id => $fi->blog_id } );
                    if ($fi->startdate) {
                        my $archiver = $pub->archiver( $at );
                        if ( my $range = $archiver->date_range ) {
                            ( $start, $end ) = $range->( $fi->startdate );
                            $entry = MT::Entry->load( { authored_on => [ $start, $end ] },
                                                      { range_incl => { authored_on => 1 }, limit => 1 } );
                        }
                    }
                    my $cat;
                    if ( $fi->category_id ) {
                        $cat = MT::Category->load( { id => $fi->category_id } );
                    }
                    my $map = MT::TemplateMap->load( { id => $fi->templatemap_id } );
                    my $file = $pub->archive_file_for( $entry, $blog, $at, $cat, $map );
                    unless ( defined($file) ) {
                        return $app->error($app->translate('Archive was not found'));
                    }
                    $map->{__saved_output_file} = $file;
                    my $ctx = MT::Template::Context->new;
                    $ctx->{current_archive_type} = $at;
                    if ( $start && $end ) {
                        $ctx->{current_timestamp} = $start;
                        $ctx->{current_timestamp_end} = $end;
                    }
                    my $arch_root = ( $at eq 'Page' ) ? $blog->site_path : $blog->archive_path;
                    my %cond;
                    $pub->rebuild_file( $blog, $arch_root, $map, $at, $ctx, \%cond, 1, FileInfo => $fi );
                }
            }
            if ( -f $fp ) {
                open( FH, $fp );
                my $html;
                while ( <FH> ){
                    $html .= $_;
                }
                close( FH );
                return $html;
            }
        }
    }
    unless ( $blog_id ) {
       $blog_id = $blog->id;
    }
    if ( my $tmpl = MT::Template->load({ blog_id => $blog_id,
                                         type => 'dynamic_error' })) {
        my $ctx = $tmpl->context;
        $ctx->stash('blog', $blog);
        $ctx->stash('blog_id', $blog_id);
        return $tmpl->output();
    }
    return $app->error($app->translate('File was not found'));
}

1;
